@@ -7,10 +7,10 @@ use Carp;
 use Scalar::Does ();
 use Scalar::Util ();
 use Try::Tiny;
+use List::MoreUtils qw/part/;
 use overload '~~' => \&_matches, '""' => \&_stringify;
 
-our $VERSION = "1.00";
-
+our $VERSION = "1.01";
 
 our $MESSAGE; # global var for last message from ~~ (see '_matches')
 
@@ -29,14 +29,36 @@ BEGIN {
 
 }
 use Sub::Exporter -setup => {
-  exports => ['node_from_path', @builtin_domains,
-              map {$_ => \&_wrap_domain} @domain_generators],
-  groups => {
-    generators => \@domain_generators,
-    builtins   => \@builtin_domains,
-   },
+  exports    => [ 'node_from_path', @builtin_domains,
+                  map {$_ => \&_wrap_domain} @domain_generators],
+  groups     => { generators => \@domain_generators,
+                  builtins   => \@builtin_domains,  },
+  collectors => { INIT => \&_sub_exporter_init },
+  installer  => \&_sub_exporter_installer,
 };
 
+# customizing Sub::Exporter to support "bang-syntax" (excluding symbols)
+# see https://rt.cpan.org/Public/Bug/Display.html?id=80234
+{ my @dont_export;
+
+  sub _sub_exporter_init {
+    my ($collection, $context) = @_;
+
+    my $args = $context->{import_args};
+    my ($exclude, $regular_args) 
+      = part {!ref $_->[0] && $_->[0] =~ /^!/ ? 0 : 1} @$args;
+    @$args = @$regular_args;
+    @dont_export = map {substr($_->[0], 1)} @$exclude;
+    1;
+  }
+
+  sub _sub_exporter_installer {
+    my ($arg, $to_export) = @_;
+    my %export_hash = @$to_export;
+    delete @export_hash{@dont_export};
+    Sub::Exporter::default_installer($arg, [%export_hash]);
+  }
+}
 
 # convenience functions : for each builtin domain, we export a closure
 # that just calls new() on the corresponding subclass. For example,
@@ -240,16 +262,16 @@ sub _call_lazy_domain {
   my ($self, $domain, $context) = @_;
 
   if ($domain && Scalar::Does::does($domain, 'CODE')) {
-    $domain = try {$domain->($context)} 
+    $domain = try   {$domain->($context)} 
               catch {# error message without "at source_file, line ..."
                      (my $error_msg = $_) =~ s/\bat\b.*//s;
-                     Data::Domain::_None->new(-name     => "domain parameters",
+                     Data::Domain::Empty->new(-name     => "domain parameters",
                                               -messages => $error_msg);
                    };
 
     Scalar::Does::does($domain, "Data::Domain")
-        or croak "lazy domain coderef returned an invalid domain";
-    }
+      or croak "lazy domain coderef returned an invalid domain";
+  }
   return $domain;
 }
 
@@ -1069,28 +1091,6 @@ sub _inspect {
 }
 
 
-
-#======================================================================
-package Data::Domain::_None; # a domain that always fails
-#======================================================================
-use strict;
-use warnings;
-use Carp;
-our @ISA = 'Data::Domain';
-
-sub new {
-  my $class   = shift;
-  my @options = ();
-  my $self    = Data::Domain::_parse_args( \@_, \@options );
-  bless $self, $class;
-  return $self;
-}
-
-sub _inspect {
-  my ($self, $data) = @_;
-  return $self->msg(INVALID => "");
-}
-
 #======================================================================
 1;
 
@@ -1123,9 +1123,24 @@ Data::Domain - Data description and validation
   my $messages = $domain->inspect($some_data);
   display_error($messages) if $messages;
 
+  # smart match API
   $some_other_data ~~ $domain
     or die "did not match because $Data::Domain::MESSAGE";
 
+  # custom name and custom messages (2 different ways)
+  $domain = Int(-name => 'age', -min => 3, -max => 18, 
+                -messages => "only for people aged 3-18");
+  $domain = Int(-name => 'age', -min => 3, -max => 18, -messages => {
+                   TOO_BIG   => "not for old people over %d",
+                   TOO_SMALL => "not for babies under %d",
+                 });
+
+  # recursive domain
+  my $expr_domain = One_of(Num, Struct(operator => String(qr(^[-+*/]$)),
+                                       left     => sub {$expr_domain},
+                                       right    => sub {$expr_domain}));
+
+
 =head1 DESCRIPTION
 
 A data domain is a description of a set of values, either
@@ -1218,7 +1233,15 @@ can be renamed in various ways. Here is an example :
     ...
   );
 
-See L<Sub::Exporter> for other renaming examples.
+See L<Sub::Exporter> for other examples of renaming imported functions.
+
+To preserve backwards compatibility with L<Exporter>, the present
+module also supports "bang-syntax" to exclude some specific symbols
+from the import list : 
+
+  use Data::Domain qw/:all !Date/;
+
+will import everything except the C<Date> function.
 
 
 =head2 Smart matching
@@ -2109,7 +2132,8 @@ L<HTML::Widget::Constraint|HTML::Widget::Constraint>,
 L<Jifty::DBI|Jifty::DBI>,
 L<Data::Constraint|Data::Constraint>,
 L<Declare::Constraints::Simple|Declare::Constraints::Simple>,
-L<Moose::Manual::Types>.
+L<Moose::Manual::Types>,
+L<Smart::Match>.
 Among those, C<Declare::Constraints::Simple> is the closest to
 C<Data::Domain>, because it is also designed to deal with
 substructures; yet it has a different approach to combinations
